#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['midgard']

package_data = \
{'': ['*'],
 'midgard': ['collections/*',
             'collections/dataset/*',
             'config/*',
             'coords/*',
             'dev/*',
             'files/*',
             'math/*',
             'parsers/*']}

install_requires = \
['numpy']

setup(name='midgard',
      version='0.1.1',
      description='Midgard, the Python Geodesy library',
      author='Norwegian Mapping Authority',
      author_email='geir.arne.hjelle@kartverket.no',
      url='https://github.com/kartverket/midgard',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='>=3.6',
     )
